#!/usr/bin/perl -w
#
# Toshiba Magnia SG20 controller v2.1 (lcd, buttons, fans)
#
# depends on: kernel sensors (i2c_viapro, via686a), fanctl, stty, cat, perl(strict, Fcntl)
#
# This is free software.  You may redistribute copies of it under the terms of
# the GNU General Public License <http://www.gnu.org/licenses/gpl.html>
# There is NO WARRANTY, to the extent permitted by law.
#
# written by ariel/KotCzarny (tjosko@yahoo-com)
# 2005-09-21, initial release, for kernel 2.4 and proprietary hw.o module
# 2007-09-15, switched to linux kernel 2.6 completely, thanks to Eric Dube for fanctl!
# 2007-09-16, fixed silly bug that kept the fans unchanged
#
# (hint for mail address: s/-/./g)
# (hint for shutdown: add this line:
#   echo >/dev/ttyS0 -en "\x1b2q Power OFF\n"
# )

use strict;
use Fcntl;

my $DEBUG=0;
my $delay=4;		# update this often (in seconds)
my $temp_hist_on=51;	# temp. to turn fans ON
my $temp_hist_off=40;	# temp. to turn fans OFF
my $fan_ctrl_cpu=1;	# if negative, don't change CPU fan status
my $fan_ctrl_sys=-1;	# if negative, don't change SYS fan status
my $blight_timeout=3600; # if negative, don't turn OFF the backlight, timeout is blight seconds
my $net_dev="eth1";	# net device to monitor activity
my $fanctl="/usr/local/sbin/fanctl"; # fanctl location
my $i2c="/sys/devices/platform/i2c-adapter/i2c-9191/9191-6000"; # location of sensors output

my @days=("Su", "Mo", "Tu", "We", "Th", "Fr", "Sa", "Su");
my @months=("Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec");

# --- configuration ends here ---

my $l1="l1"; my $l2="l2";
my $disp_mode=0; my @disp_modes=("General display", "Character table");
my $mode_shutdown=0; my $bank=0; my $tick=0;
my $ts=time; my $ots=0; my $dts=0;
my $blight=$blight_timeout;

`stty -F /dev/ttyS0 9600 cs8 cread -icanon -echo raw -crtscts`;
sysopen(F2, "/dev/ttyS0", O_RDWR);

sub round { return int($_[0]+.5); }
sub d { if($DEBUG) { print "[".join(" ",@_)."]\n"; } }

sub u { u1(); u2(); }
sub u1 { syswrite F2, "\033H$l1"; }
sub u2 { syswrite F2, "\033H\n$l2"; }

sub pad { my $tmp=substr($_[0],0,16); $tmp.=substr("                ",0,16-length($tmp)); return $tmp; }
sub p { my $x=$_[1]; my $y=$_[2]; my $tmp="\033H"; my $i=0;
 if($y>0) { $tmp.="\n"; }
 if($x>=0) { for(; $i<$x; $i++) { $tmp.="\033M"; } }
 $tmp.=substr($_[0],0,16); $l2=$tmp;
}

sub fan { # fan(c1,c2,command): c1 cpu, c2 sys, command -1 off 0 don't change 1 on
 my $c1=$_[0]; my $c2=$_[1]; my $msg='';
 if($fan_ctrl_cpu==1) {
  if($c1==0){ $msg.='FC0'; `$fanctl -c off`; }
  elsif($c1==1){ $msg.='FC1'; `$fanctl -c on`; }
 }
 if($fan_ctrl_sys==1) {
  if($c2==0){ $msg.='FS0'; `$fanctl -s off`; }
  elsif($c2==1){ $msg.='FS1'; `$fanctl -s on`; }
 }
 d($msg);
}

sub backlight { # backlight(command): command 0 off 1 on
 my $c1=$_[0]; if($c1==0){ syswrite F2, "\x1bF"; } elsif($c1==1){ syswrite F2, "\x1bb"; }
}

sub makeclock {
 my $sec; my $min; my $hour; my $mday; my $mon; my $year; my $wday; my $yday; my $isdst; my $tmp;
 ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime(time);
 # $tmp=sprintf("\033H%s %s %2d %02d:%02d", $days[$wday], $months[$mon], $mday, $hour, $min);
 # $tmp=sprintf("%02d:%02d:%02d", $hour, $min, $sec);
 if($tick) { $tmp=sprintf(chr(205)."%02d:%02d", $hour, $min); }
 else      { $tmp=sprintf(" %02d:%02d", $hour, $min); }
 return $tmp;
}

sub max { if ($_[0] > $_[1]) { return $_[0]; } else { return $_[1]; } }

$l1="\x1bXMagnia-SG20 Ctrl";  $l2=chr(207)."ariel/KotCzarny"; u(); backlight(1);

my $nfound; my $rin = ""; vec($rin,fileno(F2),1) = 1; my $rout;
my $i=0; my $button=""; my @data; my @t1; my @t2; my @load; my @mem; my @fan1; my @fan2;
my @net; my $net_rx; my $net_tx; my $nnet_rx=0; my $nnet_tx=0; my $onet_rx; my $onet_tx;

while (1){ $tick++; $tick %= 2;
 if($nfound = select($rout=$rin, "", "", $delay))
 {
  if(sysread(F2, $button, 1))
  {

   # select button pressed
if($blight!=0)
{
   if($button eq "S") {
    if($mode_shutdown>0) { $l2=pad(""); u2(); $mode_shutdown=0; }
    else { $disp_mode++; $disp_mode %= @disp_modes; $l2=pad($disp_modes[$disp_mode]); u2(); }
   }

   # power button pressed
   if($button eq "A")
   {
    if($disp_mode=="0") {
     if($mode_shutdown==0) { $l1="Press [POWER] to"; $l2="confirm shutdown"; $mode_shutdown++; u(); }
     else { $l1="    SHUTDOWN    "; $l2="  in progress!  "; u(); `poweroff`; }
    }
    elsif($disp_mode=="1") {
     $bank++; if($bank>15) { $bank=0; }
     $l2=""; for($i=0;$i<16;$i++){ $l2.=chr($bank*16+$i); } u2();
    }
   }
}
$blight=$blight_timeout; backlight(1);

  }
 }

if($mode_shutdown==0)
{
 if ($blight>0) { $blight-=$delay; if($blight<=0) { $blight=0; backlight(0); } }
 $ots=$ts; $ts=time; $dts=$ts-$ots;


 @load=`cat /proc/loadavg`; $load[0] =~ s/ .*\n//;
 $l1=sprintf("\033H%d%s %s", $disp_mode, makeclock(), $load[0]);

 if($disp_mode=="0") {
  $t1[0] = round(`cat $i2c/temp1_input`/1000); # cpu temp
  $t2[0] = round(`cat $i2c/temp2_input`/1000); # sys temp

  $fan1[0] = `cat $i2c/fan1_input`+0; # cpu rpm
  if($fan1[0]==99999) { $fan1[1]=chr(183); } else { $fan1[1]=sprintf("%1.0f", $fan1[0]/1000); }

  $fan2[0] = `cat $i2c/fan2_input`+0; # sys rpm
  if($fan2[0]==99999) { $fan2[1]=chr(183); } else { $fan2[1]=sprintf("%1.0f", $fan2[0]/1000); }

  if($t1[0]>=$temp_hist_on && ($fan1[0]==0 || $fan2[0]==0)) { fan(1,1); }
  if($t1[0]<=$temp_hist_off && $fan1[0]>0 && $fan2[0]>0) { fan(0,0); }

  #@mem=`cat /proc/meminfo`; @mem=grep(/^Mem:/, @mem); $mem[0] =~ s/.*=//;

  @net=`cat /proc/net/dev`; @net=grep(/$net_dev/, @net); $net[0] =~ s/ *$net_dev: *//;
  @net=split(/\s+/, $net[0]);
  $onet_rx=$nnet_rx; $onet_tx=$nnet_tx;
  $nnet_rx=$net[0]; $nnet_tx=$net[8];
  $net_rx=sprintf("%3.1f", ($nnet_rx-$onet_rx)/max($dts,0.00000001)/1024*0.95 ); # 0.95
  $net_tx=sprintf("%3.1f", ($nnet_tx-$onet_tx)/max($dts,0.00000001)/1024*0.95 ); # for tcp overhead
  #print "$dts [$onet_rx | $onet_tx] [$nnet_rx | $nnet_tx] [$net_rx | $net_tx]\n";

  $l2=substr(pad("$fan1[1]$fan2[1] ".chr(198)."$net_rx".chr(197)."$net_tx"),0,12)." $t2[0]".chr(178)." ";
  u2(); # print "$l2\n";

  $l1.=" $t1[0]".chr(178)."   ";
  
 }
 elsif($disp_mode=="1") { $l1.=" ".($bank*16)."   "; }
 u1();
} #if $mode_shutdown==0

}
